//
//                                                                         
// Module: Internals Example Source File                                   
//                                                                         
// Description: Declarations for the Internals Example Plugin              
//                                                                         
//                                                                         
// This source code module, and all information, data, and algorithms      
// associated with it, are part of CUBE technology (tm).                   
//                           PROPRIETARY                                   
// Copyright (c) 1996-2006 Image Space Incorporated.  All rights reserved. 
//                                                                         
//                                                                         
// Change history:                                                         
//   tag.2005.11.30: created                                               
//   jmm.2006.01.06: modified for public                                   
//                                                                         
//

//***************************************************************************//
#include "Example.hpp"          // corresponding header file
#include <math.h>               // for atan2, sqrt
#include <stdio.h>              // for sample output
//***************************************************************************//
//RacingMat
#include "serialib.h"	// Library described above
#include "ReadIni.h"
#include <tchar.h>

//create the serial communication SerialCom
serialib SerialCom;   

   char *cpMode; // grip lost or blocked wheel
   char *cpLevel; // grip loss level
   char *cpApproxDiam; // approximative wheel diameter

//***************************************************************************//
// plugin information
// Change this information to siut your needs. rFactor will use this information
// to ensure that it is using the most recent version
unsigned g_uPluginID          = 2;
//RacingMat
char     g_szPluginName[]     = "RacingMat Brake Pedal - 2013-07-12";
//RacingMat
unsigned g_uPluginVersion     = 002;
unsigned g_uPluginObjectCount = 1;
InternalsPluginInfo g_PluginInfo;
//***************************************************************************//

//***************************************************************************//
// interface to plugin information
extern "C" __declspec(dllexport)
const char* __cdecl GetPluginName() { return g_szPluginName; }

extern "C" __declspec(dllexport)
unsigned __cdecl GetPluginVersion() { return g_uPluginVersion; }

extern "C" __declspec(dllexport)
unsigned __cdecl GetPluginObjectCount() { return g_uPluginObjectCount; }

// get the plugin-info object used to create the plugin.
extern "C" __declspec(dllexport)
PluginObjectInfo* __cdecl GetPluginObjectInfo( const unsigned uIndex )
{
  switch(uIndex)
  {
    case 0:
      return  &g_PluginInfo;
    default:
      return 0;
  }
}
//***************************************************************************//

//***************************************************************************//
// InternalsPluginInfo class
InternalsPluginInfo::InternalsPluginInfo()
{
  // put together a name for this plugin
  sprintf( m_szFullName, "%s - %s", g_szPluginName, InternalsPluginInfo::GetName() );
}
//***************************************************************************//

//***************************************************************************//
const char*    InternalsPluginInfo::GetName()     const { return ExampleInternalsPlugin::GetName(); }
const char*    InternalsPluginInfo::GetFullName() const { return m_szFullName; }
// Change this to suit your needs
const char*    InternalsPluginInfo::GetDesc()     const { return "RacingMat Brake Pedal"; }
const unsigned InternalsPluginInfo::GetType()     const { return ExampleInternalsPlugin::GetType(); }
const char*    InternalsPluginInfo::GetSubType()  const { return ExampleInternalsPlugin::GetSubType(); }
const unsigned InternalsPluginInfo::GetVersion()  const { return ExampleInternalsPlugin::GetVersion(); }
void*          InternalsPluginInfo::Create()      const { return new ExampleInternalsPlugin(); }
//***************************************************************************//

//***************************************************************************//
// InternalsPlugin class
const char ExampleInternalsPlugin::m_szName[] = "RacingMatBrakePedalPlugin";
const char ExampleInternalsPlugin::m_szSubType[] = "Internals";
const unsigned ExampleInternalsPlugin::m_uID = 1;
const unsigned ExampleInternalsPlugin::m_uVersion = 1;
//***************************************************************************//

//***************************************************************************//
PluginObjectInfo *ExampleInternalsPlugin::GetInfo() { return &g_PluginInfo; }
//***************************************************************************//

//***************************************************************************//
void ExampleInternalsPlugin::Startup()
{
 // Open ports, read configs, whatever you need to do. 
 // Read the windows COM Port of the Arduino
 // in the ini file

// default enabled to true
    mEnabled = true;
    int Ret;      // Used for return values

// path of the ini file (same directory as this plugin)
   LPCSTR localPath="Plugins\\BrakePlugin.ini"; 

   char *cpComPort;
   char *cpBaudRate;


 // read a string value of a key in a section of the ini file
 //                            section        key        default value 	    ini file path
	cpComPort=ReadKeyString(_T("Serial"), _T("Port"), _T("\\\\.\\COM6"), _T(localPath));
   // Beware !! if the address is given by the ReadKey function (from ini file), no double \\ before com port address
 	cpBaudRate=ReadKeyString(_T("Serial"), _T("Baudrate"), _T("9600"), _T(localPath));
	// Open serial link Com Port at BaudRate
	Ret=SerialCom.Open(cpComPort,int(cpBaudRate)); 

 	cpMode=ReadKeyString(_T("Parameters"), _T("Mode"), _T("\\\\.\\COM6"), _T(localPath));
 	cpLevel=ReadKeyString(_T("Parameters"), _T("Level"), _T("0.6"), _T(localPath));
 	cpApproxDiam=ReadKeyString(_T("Parameters"), _T("ApproxDiam"), _T("0.5"), _T(localPath));


}
//***************************************************************************//

//***************************************************************************//
void ExampleInternalsPlugin::EnterRealtime()
{
  // start up timer every time we enter realtime
  mET = 0.0f;
}
//***************************************************************************//

//***************************************************************************//
void ExampleInternalsPlugin::UpdateTelemetry( const TelemInfo &info )
{
  const unsigned short cusBufferSize = 1024;
  char* cpTemp = new char[cusBufferSize];

    const float metersPerSec = sqrtf( ( info.mLocalVel.x * info.mLocalVel.x ) +
                                    ( info.mLocalVel.y * info.mLocalVel.y ) +
                                    ( info.mLocalVel.z * info.mLocalVel.z ) );
    //RacingMat
  int noGripWheels = 0;	
  int lockedWheels = 0;	

  float approxDiam = 0.5 ; // toDo convert char cpApproxDiam into float
  float level = 0.80; // toDo convert char cpLevel into float
//  level = 0.80 -> wheels must have less than 80% of the grip
  char* vibrate;
  int Ret;

  vibrate="0"; // false
  if ((metersPerSec>0.1) & (info.mUnfilteredBrake >= 0.1 ))
  {
  if (cpMode="1") 
    {
	for( long i = 0; i < 4; ++i )
    { // for each wheel
		// Wheels (i==0)"FrontLeft"
		// (i==1) "FrontRight"
		// (i==2) "RearLeft"
		// (i==3) "RearRight"
      const TelemWheel &wheel = info.mWheel[i];

	  // count how many wheels are blocked
	  // check if wheel rotates less than expected by the vehicule speed (minus 50%)
	  if ((3.14*approxDiam*abs(wheel.mRotation))<(0.50*metersPerSec))  { ++lockedWheels; }
	}
	  if (lockedWheels>=2)  
		 vibrate="1"; //true
  } else
    {
	for( long i = 0; i < 4; ++i )
    { // for each wheel
		// Wheels (i==0)"FrontLeft"
		// (i==1) "FrontRight"
		// (i==2) "RearLeft"
		// (i==3) "RearRight"
      const TelemWheel &wheel = info.mWheel[i];

	  // count how many wheels lost its grip
	  if (wheel.mGripFract<level) { ++noGripWheels; } // coupler avec freinage en cours et avec vitesse rel non nulle
	}
		//envoie le signal perte de grip ou non
		if (noGripWheels>=2)
		 vibrate="1"; //true
	}
  }
  Ret=SerialCom.WriteString(vibrate);       // Send the command on the serial port
	    
  if(cpTemp)
    delete [] cpTemp;
}
//***************************************************************************//

//***************************************************************************//
void ExampleInternalsPlugin::UpdateGraphics( const GraphicsInfo &info )
{
//
}
//***************************************************************************//

//***************************************************************************//
bool ExampleInternalsPlugin::CheckHWControl( const char * const controlName, float &fRetVal )
{
  return( false );
}
//***************************************************************************//

//***************************************************************************//
bool ExampleInternalsPlugin::ForceFeedback( float &forceValue )
{
  return( false );
}
//***************************************************************************//
